#ifndef LIST_MT_H
#define LIST_MT_H


#include <qvaluelist.h>
#include <qmutex.h>


template<class Type> class List_MT {

public:

    List_MT() {
    }

    ~List_MT() {

        flush();
    }

    List_MT(const List_MT<Type>& origList) {

        mutex_.lock();

        list_.clear();
        List_MT<Type> &ref = const_cast<List_MT<Type>&> (origList); // get rid of const

        QValueList<Type>::const_iterator iter;
        for (iter = ref.begin(); iter != ref.end(); ++iter)
            list_.push_back(*iter);

        mutex_.unlock();

    }

    List_MT& operator=(const List_MT<Type>& origList) {

        mutex_.lock();

        if (this != &origList) {

            list_.clear();
            List_MT<Type> &ref = const_cast<List_MT<Type>&> (origList); // get rid of const

            QValueList<Type>::const_iterator iter;
            for (iter = ref.begin(); iter != ref.end(); ++iter)
                list_.push_back(*iter);

        }

        mutex_.unlock();

        return *this;

    }

    bool isEmpty() {

        mutex_.lock();
        bool empty = list_.empty();
        mutex_.unlock();
        return empty;
    }

    int count() {

        mutex_.lock();
        int num = list_.count();
        mutex_.unlock();
        return num;
    }



    void flush() {

        mutex_.lock();
        list_.clear();
        mutex_.unlock();
    }

    void push_front(const Type& t) {

        mutex_.lock();
        list_.push_front(t);
        mutex_.unlock();
    }

    void push_back(const Type& t) {

        mutex_.lock();
        list_.push_back(t);
        mutex_.unlock();
    }

    void pop_front() {

        mutex_.lock();
        list_.pop_front();
        mutex_.unlock();
    }

    void pop_back() {

        mutex_.lock();
        list_.pop_back();
        mutex_.unlock();
    }

    Type first() {

        mutex_.lock();
        Type t(list_.first());
        mutex_.unlock();
        return t;
    }

    Type last() {

        mutex_.lock();
        Type t(list_.last());
        mutex_.unlock();
        return t;
    }

    QValueList<Type>::const_iterator begin() {

        mutex_.lock();
        QValueList<Type>::const_iterator iter;
        iter = list_.begin();
        mutex_.unlock();
        return iter;

    }

    QValueList<Type>::const_iterator end() {

        mutex_.lock();
        QValueList<Type>::const_iterator iter;
        iter = list_.end();
        mutex_.unlock();
        return iter;

    }


private:

    QValueList<Type> list_;
    QMutex mutex_;

};

#endif

